"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("./helpers");
const config_1 = require("./config");
class NGram {
    constructor(n) {
        this.n = n;
        this.experience = {};
    }
    tokensToKey(tokens) {
        return tokens.join(' ').toLowerCase();
    }
    getExperience(currentTokens) {
        const key = this.tokensToKey(currentTokens);
        return this.experience[key];
    }
    addExperience(gram) {
        const { current, next } = gram;
        const key = this.tokensToKey(current);
        const nextKey = next.toLowerCase();
        if (!this.experience[key]) {
            this.experience[key] = { candidates: {}, total: 0 };
        }
        if (!this.experience[key].candidates[nextKey]) {
            this.experience[key].candidates[nextKey] = {
                original: next,
                count: 0,
            };
        }
        this.experience[key].candidates[nextKey].count++;
        this.experience[key].total++;
    }
    getNextToken(currentTokens) {
        const experience = this.getExperience(currentTokens);
        if (!experience) {
            return false;
        }
        const { candidates, total } = experience;
        const random = Math.random();
        let probSum = 0;
        for (const candidate in candidates) {
            const prob = candidates[candidate].count / total;
            probSum += prob;
            if (probSum >= random) {
                return candidate;
            }
        }
    }
    train(str) {
        const sentences = str.replace(/\.{2,}/g, ' ').split('.');
        sentences.forEach(sentence => (0, helpers_1.getNgrams)(this.n, sentence, gram => {
            this.addExperience(gram);
        }));
    }
    guess(starter = '', length = 0) {
        if (!Object.keys(this.experience).length) {
            throw new Error('No experience, should train first');
        }
        const starterTokens = (0, helpers_1.getTokens)(starter);
        const result = Array(this.n - 1)
            .fill(config_1.START_TOKEN)
            .concat(starterTokens);
        /* eslint-disable no-unmodified-loop-condition */
        while (length === 0 ||
            result.length - (this.n - 1) - starterTokens.length < length) {
            const nextToken = this.getNextToken(result.slice(-(this.n - 1)));
            if (!nextToken) {
                break;
            }
            result.push(nextToken);
        }
        return result.slice(this.n - 1 + starterTokens.length);
    }
}
exports.default = NGram;
//# sourceMappingURL=index.js.map